@@ -16,10 +16,10 @@ import os
 import types
 import sets
 
-from exceptions import *
-from message import *
-from utilities import *
-from baseclasses import ConfigurableBase, ForkingBase
+from getmailcore.exceptions import *
+from getmailcore.message import *
+from getmailcore.utilities import *
+from getmailcore.baseclasses import *
 
 #######################################
 class FilterSkeleton(ConfigurableBase):
@@ -63,8 +63,9 @@ class FilterSkeleton(ConfigurableBase):
         try:
             self.initialize()
         except KeyError, o:
-            raise getmailConfigurationError('missing required configuration'
-                ' parameter %s' % o)
+            raise getmailConfigurationError(
+                'missing required configuration parameter %s' % o
+            )
         self.log.trace('done\n')
 
     def filter_message(self, msg, retriever):
@@ -76,18 +77,26 @@ class FilterSkeleton(ConfigurableBase):
         if exitcode in self.exitcodes_drop:
             # Drop message
             self.log.debug('filter %s returned %d; dropping message\n'
-                % (self, exitcode))
+                           % (self, exitcode))
             return None
-        elif (exitcode not in self.exitcodes_keep) or err:
+        elif (exitcode not in self.exitcodes_keep):
             raise getmailFilterError('filter %s returned %d (%s)\n'
-                % (self, exitcode, err))
+                                     % (self, exitcode, err))
+        elif err:
+            if self.conf['ignore_stderr']:
+                self.log.info('filter %s: %s\n' % (self, err))
+            else:
+                raise getmailFilterError('filter %s returned %d (%s)\n'
+                                         % (self, exitcode, err))
 
         # Check the filter was sane
         if len(newmsg.headers()) < len(msg.headers()):
             # Warn user
-            self.log.warning('Warning: filter %s returned fewer headers'
-                ' (%d) than supplied (%d)\n'
-                % (self, len(newmsg.headers()), len(msg.headers())))
+            self.log.warning(
+                'Warning: filter %s returned fewer headers (%d) than supplied '
+                '(%d)\n'
+                % (self, len(newmsg.headers()), len(msg.headers()))
+            )
 
         # Copy attributes from original message
         newmsg.copyattrs(msg)
@@ -147,50 +156,56 @@ class Filter_external(FilterSkeleton, ForkingBase):
       allow_root_commands (boolean, optional) - if set, external commands are
                                         allowed when running as root.  The
                                         default is not to allow such behaviour.
+
+      ignore_stderr (boolean, optional) - if set, getmail will not consider the
+            program writing to stderr to be an error.  The default is False.
     '''
     _confitems = (
-        {'name' : 'path', 'type' : str},
-        {'name' : 'unixfrom', 'type' : bool, 'default' : False},
-        {'name' : 'arguments', 'type' : tuple, 'default' : ()},
-        {'name' : 'exitcodes_keep', 'type' : tuple, 'default' : (0, )},
-        {'name' : 'exitcodes_drop', 'type' : tuple, 'default' : (99, 100)},
-        {'name' : 'user', 'type' : str, 'default' : None},
-        {'name' : 'group', 'type' : str, 'default' : None},
-        {'name' : 'allow_root_commands', 'type' : bool, 'default' : False},
-        {'name' : 'configparser', 'type' : types.InstanceType, 'default' : None},
+        ConfFile(name='path'),
+        ConfBool(name='unixfrom', required=False, default=False),
+        ConfTupleOfStrings(name='arguments', required=False, default="()"),
+        ConfTupleOfStrings(name='exitcodes_keep', required=False,
+                           default="(0, )"),
+        ConfTupleOfStrings(name='exitcodes_drop', required=False,
+                           default="(99, 100)"),
+        ConfString(name='user', required=False, default=None),
+        ConfString(name='group', required=False, default=None),
+        ConfBool(name='allow_root_commands', required=False, default=False),
+        ConfBool(name='ignore_stderr', required=False, default=False),
+        ConfInstance(name='configparser', required=False),
     )
 
     def initialize(self):
         self.log.trace()
-        self.conf['path'] = expand_user_vars(self.conf['path'])
         self.conf['command'] = os.path.basename(self.conf['path'])
-        if not os.path.isfile(self.conf['path']):
-            raise getmailConfigurationError('no such command %s'
-                % self.conf['path'])
         if not os.access(self.conf['path'], os.X_OK):
-            raise getmailConfigurationError('%s not executable'
-                % self.conf['path'])
+            raise getmailConfigurationError(
+                '%s not executable' % self.conf['path']
+            )
         if type(self.conf['arguments']) != tuple:
-            raise getmailConfigurationError('incorrect arguments format;'
-                ' see documentation (%s)' % self.conf['arguments'])
+            raise getmailConfigurationError(
+                'incorrect arguments format; see documentation (%s)'
+                % self.conf['arguments']
+            )
         try:
             self.exitcodes_keep = [int(i) for i in self.conf['exitcodes_keep']
-                if 0 <= int(i) <= 255]
+                                   if 0 <= int(i) <= 255]
             self.exitcodes_drop = [int(i) for i in self.conf['exitcodes_drop']
-                if 0 <= int(i) <= 255]
+                                   if 0 <= int(i) <= 255]
             if not self.exitcodes_keep:
                 raise getmailConfigurationError('exitcodes_keep set empty')
             if sets.ImmutableSet(self.exitcodes_keep).intersection(
-                    sets.ImmutableSet(self.exitcodes_drop)):
+                sets.ImmutableSet(self.exitcodes_drop)
+            ):
                 raise getmailConfigurationError('exitcode sets intersect')
         except ValueError, o:
             raise getmailConfigurationError('invalid exit code specified (%s)'
-                % o)
+                                            % o)
 
     def __str__(self):
         self.log.trace()
         return 'Filter_external %s (%s)' % (self.conf['command'],
-            self._confstring())
+                                            self._confstring())
 
     def showconf(self):
         self.log.trace()
@@ -201,7 +216,7 @@ class Filter_external(FilterSkeleton, ForkingBase):
             # Write out message with native EOL convention
             msgfile = os.tmpfile()
             msgfile.write(msg.flatten(False, False,
-                include_from=self.conf['unixfrom']))
+                                      include_from=self.conf['unixfrom']))
             msgfile.flush()
             os.fsync(msgfile.fileno())
             # Rewind
@@ -211,20 +226,22 @@ class Filter_external(FilterSkeleton, ForkingBase):
             # Set stdout and stderr to write to files
             os.dup2(stdout.fileno(), 1)
             os.dup2(stderr.fileno(), 2)
-            change_usergroup(self.log, self.conf['user'], self.conf['group'])
+            change_usergroup(None, self.conf['user'], self.conf['group'])
             args = [self.conf['path'], self.conf['path']]
             for arg in self.conf['arguments']:
                 arg = expand_user_vars(arg)
                 for (key, value) in msginfo.items():
                     arg = arg.replace('%%(%s)' % key, value)
                 args.append(arg)
-            self.log.debug('about to execl() with args %s\n' % str(args))
+            # Can't log this; if --trace is on, it will be written to the
+            # message passed to the filter.
+            #self.log.debug('about to execl() with args %s\n' % str(args))
             os.execl(*args)
         except StandardError, o:
             # Child process; any error must cause us to exit nonzero for parent
             # to detect it
             self.log.critical('exec of filter %s failed (%s)'
-                % (self.conf['command'], o))
+                              % (self.conf['command'], o))
             os._exit(127)
 
     def _filter_message(self, msg):
@@ -241,8 +258,9 @@ class Filter_external(FilterSkeleton, ForkingBase):
         # At least some security...
         if (os.geteuid() == 0 and not self.conf['allow_root_commands']
                 and self.conf['user'] == None):
-            raise getmailConfigurationError('refuse to invoke external'
-                ' commands as root by default')
+            raise getmailConfigurationError(
+                'refuse to invoke external commands as root by default'
+            )
 
         stdout = os.tmpfile()
         stderr = os.tmpfile()
@@ -261,11 +279,11 @@ class Filter_external(FilterSkeleton, ForkingBase):
         err = stderr.read().strip()
 
         self.log.debug('command %s %d exited %d\n' % (self.conf['command'],
-            childpid, exitcode))
+                                                      childpid, exitcode))
 
         newmsg = Message(fromfile=stdout)
 
-        return exitcode, newmsg, err
+        return (exitcode, newmsg, err)
 
 #######################################
 class Filter_classifier(Filter_external):
@@ -277,7 +295,7 @@ class Filter_classifier(Filter_external):
     def __str__(self):
         self.log.trace()
         return 'Filter_classifier %s (%s)' % (self.conf['command'],
-            self._confstring())
+                                              self._confstring())
 
     def showconf(self):
         self.log.trace()
@@ -297,8 +315,9 @@ class Filter_classifier(Filter_external):
         # At least some security...
         if (os.geteuid() == 0 and not self.conf['allow_root_commands']
                 and self.conf['user'] == None):
-            raise getmailConfigurationError('refuse to invoke external'
-                ' commands as root by default')
+            raise getmailConfigurationError(
+                'refuse to invoke external commands as root by default'
+            )
 
         stdout = os.tmpfile()
         stderr = os.tmpfile()
@@ -317,13 +336,13 @@ class Filter_classifier(Filter_external):
         err = stderr.read().strip()
 
         self.log.debug('command %s %d exited %d\n' % (self.conf['command'],
-            childpid, exitcode))
+                                                      childpid, exitcode))
 
         for line in [line.strip() for line in stdout.readlines()
-                if line.strip()]:
+                     if line.strip()]:
             msg.add_header('X-getmail-filter-classifier', line)
 
-        return exitcode, msg, err
+        return (exitcode, msg, err)
 
 #######################################
 class Filter_TMDA(FilterSkeleton, ForkingBase):
@@ -348,29 +367,29 @@ class Filter_TMDA(FilterSkeleton, ForkingBase):
                                 allowed when running as root.  The default is
                                 not to allow such behaviour.
 
+      ignore_stderr (boolean, optional) - if set, getmail will not consider the
+            program writing to stderr to be an error.  The default is False.
+
       conf-break - used to break envelope recipient to find EXT.  Defaults
                                 to "-".
-
     '''
     _confitems = (
-        {'name' : 'path', 'type' : str, 'default' : '/usr/local/bin/tmda-filter'},
-        {'name' : 'user', 'type' : str, 'default' : None},
-        {'name' : 'group', 'type' : str, 'default' : None},
-        {'name' : 'allow_root_commands', 'type' : bool, 'default' : False},
-        {'name' : 'conf-break', 'type' : str, 'default' : '-'},
-        {'name' : 'configparser', 'type' : types.InstanceType, 'default' : None},
+        ConfFile(name='path', default='/usr/local/bin/tmda-filter'),
+        ConfString(name='user', required=False, default=None),
+        ConfString(name='group', required=False, default=None),
+        ConfBool(name='allow_root_commands', required=False, default=False),
+        ConfBool(name='ignore_stderr', required=False, default=False),
+        ConfString(name='conf-break', required=False, default='-'),
+        ConfInstance(name='configparser', required=False),
     )
 
     def initialize(self):
         self.log.trace()
-        self.conf['path'] = expand_user_vars(self.conf['path'])
         self.conf['command'] = os.path.basename(self.conf['path'])
-        if not os.path.isfile(self.conf['path']):
-            raise getmailConfigurationError('no such command %s'
-                % self.conf['path'])
         if not os.access(self.conf['path'], os.X_OK):
-            raise getmailConfigurationError('%s not executable'
-                % self.conf['path'])
+            raise getmailConfigurationError(
+                '%s not executable' % self.conf['path']
+            )
         self.exitcodes_keep = (0, )
         self.exitcodes_drop = (99, )
 
@@ -396,37 +415,42 @@ class Filter_TMDA(FilterSkeleton, ForkingBase):
             # Set stdout and stderr to write to files
             os.dup2(stdout.fileno(), 1)
             os.dup2(stderr.fileno(), 2)
-            change_usergroup(self.log, self.conf['user'], self.conf['group'])
+            change_usergroup(None, self.conf['user'], self.conf['group'])
             args = [self.conf['path'], self.conf['path']]
             # Set environment for TMDA
             os.environ['SENDER'] = msg.sender
             os.environ['RECIPIENT'] = msg.recipient
-            os.environ['EXT'] = self.conf['conf-break'].join('@'.join(
-                msg.recipient.split('@')[:-1]).split(
-                self.conf['conf-break'])[1:])
+            os.environ['EXT'] = self.conf['conf-break'].join(
+                '@'.join(msg.recipient.split('@')[:-1]).split(
+                    self.conf['conf-break']
+                )[1:]
+            )
             self.log.trace('SENDER="%(SENDER)s",RECIPIENT="%(RECIPIENT)s"'
-                ',EXT="%(EXT)s"' % os.environ)
+                           ',EXT="%(EXT)s"' % os.environ)
             self.log.debug('about to execl() with args %s\n' % str(args))
             os.execl(*args)
         except StandardError, o:
             # Child process; any error must cause us to exit nonzero for parent
             # to detect it
             self.log.critical('exec of filter %s failed (%s)'
-                % (self.conf['command'], o))
+                              % (self.conf['command'], o))
             os._exit(127)
 
     def _filter_message(self, msg):
         self.log.trace()
         self._prepare_child()
         if msg.recipient == None or msg.sender == None:
-            raise getmailConfigurationError('TMDA requires the message envelope'
-                ' and therefore a multidrop retriever')
+            raise getmailConfigurationError(
+                'TMDA requires the message envelope and therefore a multidrop '
+                'retriever'
+            )
 
         # At least some security...
         if (os.geteuid() == 0 and not self.conf['allow_root_commands']
                 and self.conf['user'] == None):
-            raise getmailConfigurationError('refuse to invoke external'
-                ' commands as root by default')
+            raise getmailConfigurationError(
+                'refuse to invoke external commands as root by default'
+            )
 
         stdout = os.tmpfile()
         stderr = os.tmpfile()
@@ -444,6 +468,6 @@ class Filter_TMDA(FilterSkeleton, ForkingBase):
         err = stderr.read().strip()
 
         self.log.debug('command %s %d exited %d\n' % (self.conf['command'],
-            childpid, exitcode))
+                                                      childpid, exitcode))
 
-        return exitcode, msg, err
+        return (exitcode, msg, err)
